import path from 'path';
import { ImageTransformPlugin } from 'image-transform-plugin';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

/*!
 * normalize-path <https://github.com/jonschlinkert/normalize-path>
 *
 * Copyright (c) 2014-2018, Jon Schlinkert.
 * Released under the MIT License.
 */

var normalizePath = function(path, stripTrailing) {
  if (typeof path !== 'string') {
    throw new TypeError('expected path to be a string');
  }

  if (path === '\\' || path === '/') return '/';

  var len = path.length;
  if (len <= 1) return path;

  // ensure that win32 namespaces has two leading slashes, so that the path is
  // handled properly by the win32 version of path.parse() after being normalized
  // https://msdn.microsoft.com/library/windows/desktop/aa365247(v=vs.85).aspx#namespaces
  var prefix = '';
  if (len > 4 && path[3] === '\\') {
    var ch = path[2];
    if ((ch === '?' || ch === '.') && path.slice(0, 2) === '\\\\') {
      path = path.slice(2);
      prefix = '//';
    }
  }

  var segs = path.split(/[/\\]+/);
  if (stripTrailing !== false && segs[segs.length - 1] === '') {
    segs.pop();
  }
  return prefix + segs.join('/');
};

const stringify = (data) => {
    return JSON.stringify(data, null, 2);
};
class WebpackLogger {
    constructor(compilation) {
        this.compilation = compilation;
    }
    static getLogger(compilation) {
        return new WebpackLogger(compilation);
    }
    info(msg) {
        if (typeof msg !== "string") {
            msg = stringify(msg);
        }
        console.log(msg);
    }
    warn(msg) {
        if (typeof msg !== "string") {
            msg = stringify(msg);
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        this.compilation.warnings.push(msg);
    }
    error(msg) {
        if (typeof msg !== "string") {
            msg = stringify(msg);
        }
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        this.compilation.errors.push(msg);
    }
}

class ImageTransformWebpackPlugin extends ImageTransformPlugin {
    constructor(configs, option) {
        super(configs, option);
    }
    getContext() {
        return this.webpackContext ? this.webpackContext : "./";
    }
    getLogger() {
        return this.webpackLogger;
    }
    /**
     * 仅用于webpack plugin自调用
     * @param complier
     */
    apply(compiler) {
        const ID = "ImageTransformWebpackPlugin";
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        compiler.hooks.thisCompilation.tap(ID, (compilation) => {
            const logger = WebpackLogger.getLogger(compilation);
            this.webpackContext = compiler.context;
            this.webpackLogger = logger;
            this.compiler = compiler;
            this.compilation = compilation;
            try {
                this.transform();
            }
            catch (error) {
                logger.error(error);
            }
        });
    }
    store(sharpData, output, config) {
        return __awaiter(this, void 0, void 0, function* () {
            const { compilation, compiler } = this;
            const filePath = output.file;
            const existingAsset = compilation.getAsset(filePath);
            const { RawSource } = compiler.webpack.sources;
            const context = this.getContext();
            const absoluteFilename = path.resolve(context, config.input);
            const sourceFilename = normalizePath(path.relative(context, absoluteFilename));
            return sharpData
                .toBuffer()
                .then((data) => {
                const source = new RawSource(data);
                if (existingAsset) {
                    compilation.updateAsset(filePath, source, {
                        sourceFilename,
                    });
                }
                else {
                    compilation.emitAsset(filePath, source, {
                        sourceFilename,
                    });
                }
                return true;
            })
                .catch((err) => {
                const logger = this.getLogger();
                logger.error(err);
                return true;
            });
        });
    }
}

export { ImageTransformWebpackPlugin };
